@@ -15,19 +15,10 @@ from os import access, R_OK
 from os.path import isfile
 
 from ruamel.yaml.comments import CommentedSet
-# pylint: disable=wrong-import-position,ungrouped-imports
-from ruamel.yaml import version_info as ryversion
-if ryversion < (0, 17, 22):                   # pragma: no cover
-    from yamlpath.patches.timestamp import (
-        AnchoredTimeStamp,
-        AnchoredDate,
-    )  # type: ignore
-else:                                         # pragma: no cover
-    # Temporarily fool MYPY into resolving the future-case imports
-    from ruamel.yaml.timestamp import TimeStamp as AnchoredTimeStamp
-    AnchoredDate = AnchoredTimeStamp
-    #from ruamel.yaml.timestamp import AnchoredTimeStamp
-    # From whence shall come AnchoredDate?
+from yamlpath.patches.timestamp import (
+    AnchoredTimeStamp,
+    AnchoredDate,
+)
 
 from yamlpath import __version__ as YAMLPATH_VERSION
 from yamlpath.common import Parsers, Nodes
